package java.util;

import java.security.NoSuchAlgorithmException;
import java.security.MessageDigest;
import java.security.SecureRandom;
import java.io.Serializable;

public final class UUID implements Serializable, Comparable<UUID>
{
    private static final long serialVersionUID = -4856846361193249489L;
    private final long mostSigBits;
    private final long leastSigBits;
    
    private UUID(byte[] array) {
        long mostSigBits = 0L;
        long leastSigBits = 0L;
        assert array.length == 16 : "data must be 16 bytes in length";
        for (int i = 0; i < 8; ++i) {
            mostSigBits = (mostSigBits << 8 | (long)(array[i] & 0xFF));
        }
        for (int j = 8; j < 16; ++j) {
            leastSigBits = (leastSigBits << 8 | (long)(array[j] & 0xFF));
        }
        this.mostSigBits = mostSigBits;
        this.leastSigBits = leastSigBits;
        return;
    }
    
    public UUID(long mostSigBits, long leastSigBits) {
        this.mostSigBits = mostSigBits;
        this.leastSigBits = leastSigBits;
    }
    
    public static UUID randomUUID() {
        SecureRandom numberGenerator = UUID.Holder.numberGenerator;
        byte[] bytes = new byte[16];
        numberGenerator.nextBytes(bytes);
        int n = 6;
        bytes[n] &= 0xF;
        int n2 = 6;
        bytes[n2] |= 0x40;
        int n3 = 8;
        bytes[n3] &= 0x3F;
        int n4 = 8;
        bytes[n4] |= 0x80;
        return new UUID(bytes);
    }
    
    public static UUID nameUUIDFromBytes(byte[] input) {
        MessageDigest instance;
        try {
            instance = MessageDigest.getInstance("MD5");
        }
        catch (NoSuchAlgorithmException cause) {
            throw new InternalError("MD5 not supported", cause);
        }
        byte[] digest = instance.digest(input);
        int n = 6;
        digest[n] &= 0xF;
        int n2 = 6;
        digest[n2] |= 0x30;
        int n3 = 8;
        digest[n3] &= 0x3F;
        int n4 = 8;
        digest[n4] |= 0x80;
        return new UUID(digest);
    }
    
    public static UUID fromString(String str) {
        String[] split = str.split("-");
        if (split.length != 5) {
            throw new IllegalArgumentException("Invalid UUID string: " + str);
        }
        for (int i = 0; i < 5; ++i) {
            split[i] = "0x" + split[i];
        }
        return new UUID(((long)Long.decode(split[0]) << 16 | (long)Long.decode(split[1])) << 16 | (long)Long.decode(split[2]), (long)Long.decode(split[3]) << 48 | (long)Long.decode(split[4]));
    }
    
    public long getLeastSignificantBits() {
        return this.leastSigBits;
    }
    
    public long getMostSignificantBits() {
        return this.mostSigBits;
    }
    
    public int version() {
        return (int)(this.mostSigBits >> 12 & 0xFL);
    }
    
    public int variant() {
        return (int)(this.leastSigBits >>> (int)(64L - (this.leastSigBits >>> 62)) & this.leastSigBits >> 63);
    }
    
    public long timestamp() {
        if (this.version() != 1) {
            throw new UnsupportedOperationException("Not a time-based UUID");
        }
        return (this.mostSigBits & 0xFFFL) << 48 | (this.mostSigBits >> 16 & 0xFFFFL) << 32 | this.mostSigBits >>> 32;
    }
    
    public int clockSequence() {
        if (this.version() != 1) {
            throw new UnsupportedOperationException("Not a time-based UUID");
        }
        return (int)((this.leastSigBits & 0x3FFF000000000000L) >>> 48);
    }
    
    public long node() {
        if (this.version() != 1) {
            throw new UnsupportedOperationException("Not a time-based UUID");
        }
        return this.leastSigBits & 0xFFFFFFFFFFFFL;
    }
    
    @Override
    public String toString() {
        return digits(this.mostSigBits >> 32, 8) + "-" + digits(this.mostSigBits >> 16, 4) + "-" + digits(this.mostSigBits, 4) + "-" + digits(this.leastSigBits >> 48, 4) + "-" + digits(this.leastSigBits, 12);
    }
    
    private static String digits(long n, int n2) {
        long n3 = 1L << n2 * 4;
        return Long.toHexString(n3 | (n & n3 - 1L)).substring(1);
    }
    
    @Override
    public int hashCode() {
        long n = this.mostSigBits ^ this.leastSigBits;
        return (int)(n >> 32) ^ (int)n;
    }
    
    @Override
    public boolean equals(Object o) {
        if (null == o || o.getClass() != UUID.class) {
            return false;
        }
        UUID uuid = (UUID)o;
        return this.mostSigBits == uuid.mostSigBits && this.leastSigBits == uuid.leastSigBits;
    }
    
    @Override
    public int compareTo(UUID uuid) {
        return (this.mostSigBits < uuid.mostSigBits) ? -1 : ((this.mostSigBits > uuid.mostSigBits) ? 1 : ((this.leastSigBits < uuid.leastSigBits) ? -1 : ((this.leastSigBits > uuid.leastSigBits) ? 1 : 0)));
    }
    
    private static class Holder
    {
        static final SecureRandom numberGenerator;
        
        static {
            numberGenerator = new SecureRandom();
        }
    }
}
